package geniusweb.protocol.partyconnection;

import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import geniusweb.actions.PartyId;
import geniusweb.party.inform.Inform;

/**
 * Contains all parties with their connections. immutable
 */
public class ProtocolToPartyConnections implements Iterable<ProtocolToPartyConn> {
	private List<ProtocolToPartyConn> connections;

	public ProtocolToPartyConnections(List<ProtocolToPartyConn> connections) {
		this.connections = connections;
	}

	/**
	 * @param id the {@link PartyId} needed
	 * @return a connection with that party, or null if no such connection.
	 */
	public ProtocolToPartyConn get(PartyId id) {
		return connections.stream().filter(conn -> id.equals(conn.getParty()))
				.findAny().orElse(null);
	}

	/**
	 * 
	 * @return true iff all parties have one connection only
	 */
	public boolean allunique() {
		return getParties().size() == connections.size();
	}

	/**
	 * 
	 * @return list of parties currently in the connections.
	 */
	private List<PartyId> getParties() {
		return connections.stream().map(conn -> conn.getParty()).distinct()
				.collect(Collectors.toList());
	}

	/**
	 * Broadcast info to all parties
	 * 
	 * @param info the {@link Inform} to broadcast
	 */
	public void broadcast(Inform info) throws IOException {
		for (ProtocolToPartyConn conn : connections) {
			conn.send(info);
		}
	}

	/**
	 * 
	 * @return number of connections available
	 */
	public int size() {
		return connections.size();
	}

	/**
	 * @param i
	 * @return the ith connection
	 */
	public ProtocolToPartyConn get(int i) {
		return connections.get(i);
	}

	public Stream<ProtocolToPartyConn> stream() {
		return connections.stream();
	}

	@Override
	public Iterator<ProtocolToPartyConn> iterator() {
		return connections.iterator();
	}

	public ProtocolToPartyConnections with(ProtocolToPartyConn conn) {
		List<ProtocolToPartyConn> newconnections = new LinkedList<>(
				connections);
		newconnections.add(conn);
		return new ProtocolToPartyConnections(newconnections);
	}

	@Override
	public String toString() {
		return "ConnectionWithParties" + connections;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((connections == null) ? 0 : connections.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ProtocolToPartyConnections other = (ProtocolToPartyConnections) obj;
		if (connections == null) {
			if (other.connections != null)
				return false;
		} else if (!connections.equals(other.connections))
			return false;
		return true;
	}

}
